package com.monkeycoder.monkeynes.base;

/**
 * This class provdes several math related convenience constructs.
 * <br><br>
 * Originally part of the MonkeyNes project.
 *
 * @author David Stephens
 */
public class MathHelper
{
  /**
   * This array allows access to individual bits.
   */
  public static final int[] bitMask  = { 0x00000001, 0x00000002, 0x00000004, 0x00000008,   // nibble 0 - byte 0
                                         0x00000010, 0x00000020, 0x00000040, 0x00000080,   // nibble 1 - byte 0
                                         0x00000100, 0x00000200, 0x00000400, 0x00000800,   // nibble 2 - byte 1
                                         0x00001000, 0x00002000, 0x00004000, 0x00008000,   // nibble 3 - byte 1
                                         0x00010000, 0x00020000, 0x00040000, 0x00080000,   // nibble 4 - byte 2
                                         0x00100000, 0x00200000, 0x00400000, 0x00800000,   // nibble 5 - byte 2
                                         0x01000000, 0x02000000, 0x04000000, 0x08000000,   // nibble 6 - byte 3
                                         0x10000000, 0x20000000, 0x40000000, 0x80000000    // nibble 7 - byte 3
                                       };

  /**
   * This array allows access to individual nibbles.
   */
  public static final int[] nibbleMask = { 0x0000000F, 0x000000F0, 0x00000F00, 0x0000F000,  // byte 0 and byte 1
                                           0x000F0000, 0x00F00000, 0x0F000000, 0xF0000000   // byte 2 and byte 3
                                         };

  /**
   * This array allows access to individual bytes.
   */
  public static final int[] byteMask = { 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000 };


  private MathHelper()
  {
    // Do nothing
  }

  /**
   * Forces an address to stay within the 64k range by wrapping it.
   * @param addr The address to access.
   * @return The wrapped address.
   */
  public static int getWrappedAddress( int addr )
  {
    return (addr % 0x00010000);
  }

  /**
   * Forces a byte to stay within the 0-255 range by wrapping it.
   * @param addr The address to access.
   * @return The wrapped address.
   */
  public static int getWrappedByte( int addr )
  {
    return (addr % 0x00000100);
  }

  /**
   * Builds a 0 padded 32-bit hexidecimal string from an integer.
   * @param inInt The number to convert into a hexidecimal string.
   * @return The converted string.
   */
  public static String getHexString( int inInt )
  {
    String out     = null;

    out = Integer.toString( inInt, 16 ).toUpperCase();

    if( inInt >= 0             && inInt < 16 )
      out = "0000000" + out;
    else if( inInt >= 16       && inInt < 256 )
      out = "000000" + out;
    else if( inInt >= 256      && inInt < 4096 )
      out = "00000" + out;
    else if( inInt >= 4096     && inInt < 65536 )
      out = "0000" + out;
    else if( inInt >= 65536    && inInt < 1048576 )
      out = "000" + out;
    else if( inInt >= 1048576  && inInt < 16777216 )
      out = "00" + out;
    else if( inInt >= 16777216 && inInt < 268435456 )
      out = "0" + out;

    return out;
  }

  /**
   * Builds a 0 padded 8-bit hexidecimal string from an integer.
   * @param inInt The number to convert into a hexidecimal string.
   * @return The converted string.
   */
  public static String getHexString8( int inInt )
  {
    String out     = null;

    inInt &= 0x000000FF;
    out = Integer.toString( inInt, 16 ).toUpperCase();

    if( inInt >= 0             && inInt < 16 )
      out = "0" + out;

    return out;
  }

  /**
   * Builds a 0 padded 16-bit hexidecimal string from an integer.
   * @param inInt The number to convert into a hexidecimal string.
   * @return The converted string.
   */
  public static String getHexString16( int inInt )
  {
    String out     = null;

    inInt &= 0x0000FFFF;
    out = Integer.toString( inInt, 16 ).toUpperCase();

    if( inInt >= 0             && inInt < 16 )
      out = "000" + out;
    else if( inInt >= 16       && inInt < 256 )
      out = "00" + out;
    else if( inInt >= 256      && inInt < 4096 )
      out = "0" + out;

    return out;
  }

}