package com.monkeycoder.monkeynes.gui;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.event.*;

/**
 * This class creates ButtonMaps based on values found in the <code>monkeynes.ini</code> properties file.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class ButtonMapFactory
{
  private Properties prop = new Properties();

  /**
   * Creates a new ButtonMapFactory and initializes itself with values found in 
   * the <code>monkeynes.ini</code> properties file.
   */
  public ButtonMapFactory()
  {
    try
    {
      URL iniFile = ClassLoader.getSystemResource( "monkeynes.ini" );
      if( iniFile != null )
      {
        FileInputStream fis = new FileInputStream( iniFile.getPath() );
        prop.load( fis );
        fis.close();
      }
      else
        DebugLog.log( 1, "Failed to load properties." );
    }
    catch( IOException ioe )
    {
      DebugLog.log( 1, "Failed to load properties." );
    }  
  }
  
   /**
   * Gets a ButtonMap for a specific type of controller plugged into a specific port.
   * @param inMapType The controller type. Expects types as defined in <code>NesController</code>.
   * @param inPort The joystick port this controller is plugged into.   
   * @return The requested ButtonMap.
   * @see com.monkeycoder.monkeynes.hardware.NesController
   */
  public ButtonMap getButtonMap( int inMapType, int inPort )
  {
    switch( inMapType )
    {
      case NesController.TYPE_JOYPAD:
        return getJoypadMap( inPort );
      default:
        return null;
    }
  }
  
  private ButtonMap getJoypadMap( int inPort )
  {
    ButtonMap bm = new ButtonMap();
    
    bm.addMapping( Integer.parseInt( prop.getProperty( "joypad_" + String.valueOf( inPort ) + "_button_a",          String.valueOf( KeyEvent.VK_X ) ), 10 ),     NesJoypad.BUTTON_A );
    bm.addMapping( Integer.parseInt( prop.getProperty( "joypad_" + String.valueOf( inPort ) + "_button_b",          String.valueOf( KeyEvent.VK_Z ) ), 10 ),     NesJoypad.BUTTON_B );
    bm.addMapping( Integer.parseInt( prop.getProperty( "joypad_" + String.valueOf( inPort ) + "_button_select",     String.valueOf( KeyEvent.VK_S ) ), 10 ),     NesJoypad.BUTTON_SELECT );
    bm.addMapping( Integer.parseInt( prop.getProperty( "joypad_" + String.valueOf( inPort ) + "_button_start",      String.valueOf( KeyEvent.VK_A ) ), 10 ),     NesJoypad.BUTTON_START );
    bm.addMapping( Integer.parseInt( prop.getProperty( "joypad_" + String.valueOf( inPort ) + "_button_dpad_up",    String.valueOf( KeyEvent.VK_UP ) ), 10 ),    NesJoypad.BUTTON_DPAD_UP );
    bm.addMapping( Integer.parseInt( prop.getProperty( "joypad_" + String.valueOf( inPort ) + "_button_dpad_down",  String.valueOf( KeyEvent.VK_DOWN ) ), 10 ),  NesJoypad.BUTTON_DPAD_DOWN );
    bm.addMapping( Integer.parseInt( prop.getProperty( "joypad_" + String.valueOf( inPort ) + "_button_dpad_left",  String.valueOf( KeyEvent.VK_LEFT ) ), 10 ),  NesJoypad.BUTTON_DPAD_LEFT );
    bm.addMapping( Integer.parseInt( prop.getProperty( "joypad_" + String.valueOf( inPort ) + "_button_dpad_right", String.valueOf( KeyEvent.VK_RIGHT ) ), 10 ), NesJoypad.BUTTON_DPAD_RIGHT );
    
    DebugLog.log( bm.toString() );
    
    return bm;    
  }
}