package com.monkeycoder.monkeynes.gui;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

import java.awt.event.*;

/**
 * This class converts key events into the appropriate button presses on a controller.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class NesControllerKeyListener implements KeyListener
{
  NesController pad1  = null;
  NesController pad2  = null;
  ButtonMap     map1  = null;
  ButtonMap     map2  = null;
  boolean       power = false;

  /**
   * Creates a new NesControllerKeyListener and tells it what controllers are in the 2 controller ports.
   * @param inPad1 The controller plugged into port 1.
   * @param inPad2 The controller plugged into port 2.
   */
  public NesControllerKeyListener( NesController inPad1, NesController inPad2 )
  {
    ButtonMapFactory bmf = new ButtonMapFactory();
    
    pad1 = inPad1;
    pad2 = inPad2;
    map1 = bmf.getButtonMap( pad1.getType(), 1 );
    map2 = bmf.getButtonMap( pad2.getType(), 2 );
  }

  /**
   * Called to tell the key listener whether power is on or off.
   * @param inPower The new power state for this listener.
   */
  public void setPowerOn( boolean inPower )
  {
    power = inPower;
  }

  // KeyListener Interface ----------------------------------------------------
  /**
   * Called when a key presseed event occurs.
   * @param ke The key event that just happened.
   */  
  public void keyPressed( KeyEvent ke )
  {                
    if( power )
    {
      pad1.setButtonValue( map1.getButton( ke.getKeyCode() ), 1 );
      pad2.setButtonValue( map2.getButton( ke.getKeyCode() ), 1 );
      
      DebugLog.log( "Pad1[press] - Key: " + ke.getKeyCode() + " Button: " + map1.getButton( ke.getKeyCode() ) );
      DebugLog.log( "Pad2[press] - Key: " + ke.getKeyCode() + " Button: " + map2.getButton( ke.getKeyCode() ) );
    }  
    else
      DebugLog.log( "Power off...  Keystroke ignored." );
  }

  /**
   * Called when a key released event occurs.
   * @param ke The key event that just happened.
   */  
  public void keyReleased( KeyEvent ke )
  {
    if( power )
    {
      pad1.setButtonValue( map1.getButton( ke.getKeyCode() ), 0 );
      pad2.setButtonValue( map2.getButton( ke.getKeyCode() ), 0 );
      
      DebugLog.log( "Pad1[release] - Key: " + ke.getKeyCode() + " Button: " + map1.getButton( ke.getKeyCode() ) );
      DebugLog.log( "Pad2[release] - Key: " + ke.getKeyCode() + " Button: " + map2.getButton( ke.getKeyCode() ) );
    }  
    else
      DebugLog.log( "Power off...  Keystroke ignored." );
  }

  /**
   * Called when a key typed event occurs.
   * @param ke The key event that just happened.
   */  
  public void keyTyped( KeyEvent ke )
  {
    // I think we'll probably just ignore these
  }
}