package com.monkeycoder.monkeynes.hardware;

import java.util.*;

/**
 * This class acts like the Game Genie (&trade; of Galoob) game enhancement device.
 * <br><br>
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class GameGenie implements MemoryFilter
{
  private NesMainMemory mainMem     = null;
  private ArrayList     activeCodes = new ArrayList();

  /**
   * Creates a new GameGenie with no active codes.
   */
  public GameGenie( NesMainMemory inMainMem )
  {
    mainMem = inMainMem;
  }

  /** 
   * This function will mutate values read from memory according to 
   * the active GameGenieCode rules.
   * @param inAddress The address of the requested value.
   * @return The filtered value at that address.
   */
  public int getFilteredReadValue( int inAddress )
  {
    return mainMem.readNoMessage( inAddress );
  }

  /** 
   * This function is not used by the GameGenie so it will always
   * return the value passed in with no further processing.
   * @param inAddress The address of the requested value.
   * @param inValue The value that was going to be written.
   * @return The filtered value to write at the indicated address.
   */
  public int getFilteredWriteValue( int inAddress, int inValue )
  {
    return inValue;
  }
}