package com.monkeycoder.monkeynes.hardware;

/**
 * This interface defines how classes that are used for manipulating 
 * memory on the fly must behave.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public interface MemoryFilter
{
  /** 
   * This function will be called when memory at the requested
   * address is read.  The value returned is implementation specific
   * and may be the value as found at the requested address or any 
   * other value.
   * @param inAddress The address of the requested value.
   * @return The filtered value at that address.
   */
  public int getFilteredReadValue( int inAddress );
  
  /** 
   * This function will be called when memory at the requested
   * address is written.  The value returned is implementation 
   * specific and may be the value supplied to be written or 
   * any other value.
   * @param inAddress The address of the requested value.
   * @param inValue The value that was going to be written.
   * @return The filtered value to write at the indicated address.
   */
  public int getFilteredWriteValue( int inAddress, int inValue );
}  