package com.monkeycoder.monkeynes.hardware;

import com.monkeycoder.monkeynes.base.*;

import java.io.*;

/**
 * This is a helper class for loading a NesRom from the file system.  It reads
 * all the data in from the file and creates a NesRom object.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class NesRomLoader
{  
  /**
   * Reads all the data in from a ROM file and creates a NesRom object.
   * It is assumed that the ROM will be in the INES ROM file format.
   * @param inRomFileName The path to the ROM file.
   * @return The NesRom loaded from the file.
   */
  public static NesRom loadRomFile( String inRomFileName )
  {
    NesRom  nr     = new NesRom();
    boolean failed = false;
    
    nr.setRomFileName( inRomFileName );
    
    try
    {    
      FileInputStream fis = new FileInputStream( inRomFileName );
    
      // Determine rom format...
      // For now assume INES format roms
      nr.setRomFormat( NesRom.ROM_FORMAT_INES );
      
      // How big is my file?
      File fileInfo = new File( inRomFileName );
      
      // Grab the header
      nr.setHeaderData( new byte[nr.getRomFormatHeaderSize(nr.getRomFormat())] );
      fis.read( nr.getHeaderData() );
      
      // Grab the rom data
      int[] romData = new int[(int)fileInfo.length() - nr.getRomFormatHeaderSize(nr.getRomFormat())];
      nr.setRomData( romData );

      for( int i = 0; i < romData.length; i++ )
        romData[i] = fis.read();
    }
    catch( IOException ioe )
    {
      DebugLog.log( "IO Exception" );
      failed = true;
    }
        
    if( !failed )
      return nr;
    else
      return null;
  }

}