package com.monkeycoder.monkeynes.hardware;

import com.monkeycoder.monkeynes.base.*;

/**
 * This represents a single graphics tile from the pattern tables.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class PatternTableTile
{
  private int top[]    = null;  // 8 bytes
  private int bot[]    = null;  // 8 bytes
  
  /**
   * Creates a new PatternTableTile with the specified data.  Data for this 
   * object is in the form of 1-bit per pixel and is found as layers stacked 
   * up on top of one another.  A pattern table tile has only 2 layers.
   * Additional layers are from the name tables or other places.
   * @param inTop An array of 8 bytes of data representing the top layer.
   * @param inBottom An array of 8 bytes of data representing the bottom layer.
   */
  public PatternTableTile( int[] inTop, int[] inBottom )
  {
    top = inTop;
    bot = inBottom;
  }

  /**
   * This gets the pixel value of of a requested pixel.  This value is the value
   * that results from combining the top and bottom layers.  X and Y coordinates 
   * are in typical computer graphics orientation.  +X to the right and +Y down.
   * @param x The X coordinate of the requested pixel.
   * @param y The Y coordinate of the requested pixel.
   */
  public int getPixelValue( int x, int y )
  {
    // Flip over the X
    x = 7 - x;
    
    int topBit = (top[y] & MathHelper.bitMask[x]) >> x;
    int botBit = (bot[y] & MathHelper.bitMask[x]) >> x;
  
    return (int)((topBit << 1) | (botBit));  
  }

}
