package com.monkeycoder.monkeynes.hardware.mapper;

import com.monkeycoder.monkeynes.base.*;
import com.monkeycoder.monkeynes.hardware.*;

/**
 * This is an implementation of Mapper 0.  Mapper 0 simply means 
 * that there is no mapper in use.
 * <br><br> 
 * Originally part of the MonkeyNES project.
 *
 * @author David Stephens
 */
public class MemoryMapperZero extends MemoryMapper
{
  /**
   * Creates a new MemoryMapperZero.
   * @param inMainMem Main memory bank accessed by this mapper.
   * @param inVidMem Video memory bank accessed by this mapper.
   * @param inRom ROM accessed by this mapper.
   */  
  public MemoryMapperZero( NesMainMemory inMainMem, NesVideoMemory inVidMem, NesRom inRom )
  {
    super( inMainMem, inVidMem, inRom );
  }

  public int getMapperNumber()
  {
    return 0;
  }
  
  public String getMapperName()
  {
    return "Mapper Zero";
  }

  public String getMapperDesc()
  {
    return "This mapper is used when there is no mapper.";
  }

  public void powerOn()
  {
    InesRomHeader iNesHead = rom.getInesRomHeader();
    
    // If the PRG-ROM size is 32k, just load it into $8000
    // If the PRG-ROM size is 16k, double load it to $8000 and $C000
    if( iNesHead.getPrgRomSize() == InesRomHeader.PRG_PAGE_SIZE )
    {
      DebugLog.log( rom.getPrgRomPage(0).length + " bytes of PRG-ROM loaded at Main:$8000" );
      mainMem.write( 0x00008000, rom.getPrgRomPage(0) );
      DebugLog.log( rom.getPrgRomPage(0).length + " bytes of PRG-ROM loaded at Main:$C000" );
      mainMem.write( 0x0000C000, rom.getPrgRomPage(0) );
    }
    else if( iNesHead.getPrgRomSize() == (InesRomHeader.PRG_PAGE_SIZE * 2) )
    {
      DebugLog.log( (rom.getPrgRomPage(0).length * 2) + " bytes of PRG-ROM loaded at Main:$8000" );
      mainMem.write( 0x00008000, rom.getPrgRomPage(0) );
      mainMem.write( 0x0000C000, rom.getPrgRomPage(1) );
    }
    
    // Load the CHR-ROM if there is any
    if( iNesHead.getChrRomSize() == InesRomHeader.CHR_PAGE_SIZE )
    {
      DebugLog.log( rom.getChrRomPage(0).length + " bytes of CHR-ROM loaded at Vid:$0000" );
      vidMem.write( 0x00000000, rom.getChrRomPage(0) );
    }
  }

  public void powerOff()
  {
    // Do nothing...
  }

  public void doCycle()
  {
    // Do nothing...
  }
}